@@ -2,7 +2,7 @@
  * libudffs.h
  *
  * Copyright (c) 2001-2002  Ben Fennema
- * Copyright (c) 2014-2019  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (c) 2014-2021  Pali Rohár <pali.rohar@gmail.com>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -52,7 +52,6 @@
 #define FLAG_STRATEGY4096		0x00000100
 #define FLAG_BLANK_TERMINAL		0x00000200
 
-#define FLAG_MIN_300_BLOCKS		0x00000400
 #define FLAG_CLOSED			0x00000800
 #define FLAG_VAT			0x00001000
 
@@ -140,6 +139,9 @@ struct udf_disc
 
 	struct sparingTable		*udf_stable[4];
 
+	uint32_t			start_block;
+	uint32_t			last_block;
+
 	uint32_t			vat_block;
 	uint32_t			*vat;
 	uint64_t			vat_entries;
@@ -220,6 +222,7 @@ struct udf_extent *prev_extent(struct udf_extent *, enum udf_space_type);
 uint32_t prev_extent_size(struct udf_extent *, enum udf_space_type, uint32_t, uint32_t);
 struct udf_extent *find_extent(struct udf_disc *, uint32_t);
 struct udf_extent *set_extent(struct udf_disc *, enum udf_space_type, uint32_t,uint32_t);
+void remove_extent(struct udf_disc *, struct udf_extent *);
 struct udf_desc *next_desc(struct udf_desc *, uint16_t);
 struct udf_desc *find_desc(struct udf_extent *, uint32_t);
 struct udf_desc *set_desc(struct udf_extent *, uint16_t, uint32_t, uint32_t, struct udf_data *);
