@@ -2,7 +2,7 @@
  * file.c
  *
  * Copyright (c) 2001-2002  Ben Fennema
- * Copyright (c) 2016-2019  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (c) 2016-2021  Pali Rohár <pali.rohar@gmail.com>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -739,7 +739,7 @@ struct udf_desc *udf_create(struct udf_disc *disc, struct udf_extent *pspace, co
 		}
 		if (disc->flags & FLAG_STRATEGY4096)
 		{
-			efe->icbTag.strategyType = cpu_to_le16(4096);
+			efe->icbTag.strategyType = cpu_to_le16(ICBTAG_STRATEGY_TYPE_4096);
 			efe->icbTag.strategyParameter = cpu_to_le16(1);
 			efe->icbTag.numEntries = cpu_to_le16(2);
 		}
@@ -773,7 +773,7 @@ struct udf_desc *udf_create(struct udf_disc *disc, struct udf_extent *pspace, co
 		}
 		if (filetype == ICBTAG_FILE_TYPE_DIRECTORY)
 			query_lvidiu(disc)->numDirs = cpu_to_le32(le32_to_cpu(query_lvidiu(disc)->numDirs)+1);
-		else if (filetype != ICBTAG_FILE_TYPE_STREAMDIR && filetype != ICBTAG_FILE_TYPE_VAT20 && filetype != ICBTAG_FILE_TYPE_UNDEF && !(flags & ICBTAG_FLAG_STREAM))
+		else if (filetype != ICBTAG_FILE_TYPE_STREAMDIR && filetype != ICBTAG_FILE_TYPE_VAT20 && filetype != ICBTAG_FILE_TYPE_VAT15 && !(flags & ICBTAG_FLAG_STREAM))
 			query_lvidiu(disc)->numFiles = cpu_to_le32(le32_to_cpu(query_lvidiu(disc)->numFiles)+1);
 	}
 	else
@@ -801,7 +801,7 @@ struct udf_desc *udf_create(struct udf_disc *disc, struct udf_extent *pspace, co
 		}
 		if (disc->flags & FLAG_STRATEGY4096)
 		{
-			fe->icbTag.strategyType = cpu_to_le16(4096);
+			fe->icbTag.strategyType = cpu_to_le16(ICBTAG_STRATEGY_TYPE_4096);
 			fe->icbTag.strategyParameter = cpu_to_le16(1);
 			fe->icbTag.numEntries = cpu_to_le16(2);
 		}
@@ -835,7 +835,7 @@ struct udf_desc *udf_create(struct udf_disc *disc, struct udf_extent *pspace, co
 		}
 		if (filetype == ICBTAG_FILE_TYPE_DIRECTORY)
 			query_lvidiu(disc)->numDirs = cpu_to_le32(le32_to_cpu(query_lvidiu(disc)->numDirs)+1);
-		else if (filetype != ICBTAG_FILE_TYPE_STREAMDIR && filetype != ICBTAG_FILE_TYPE_VAT20 && filetype != ICBTAG_FILE_TYPE_UNDEF && !(flags & ICBTAG_FLAG_STREAM))
+		else if (filetype != ICBTAG_FILE_TYPE_STREAMDIR && filetype != ICBTAG_FILE_TYPE_VAT20 && filetype != ICBTAG_FILE_TYPE_VAT15 && !(flags & ICBTAG_FLAG_STREAM))
 			query_lvidiu(disc)->numFiles = cpu_to_le32(le32_to_cpu(query_lvidiu(disc)->numFiles)+1);
 	}
 
