@@ -2,7 +2,7 @@
  * main.c
  *
  * Copyright (c) 2001-2002  Ben Fennema
- * Copyright (c) 2014-2018  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (c) 2014-2021  Pali Rohár <pali.rohar@gmail.com>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <inttypes.h>
 #include <locale.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -54,7 +55,7 @@ static int valid_offset(int fd, off_t offset)
 
 	if (lseek(fd, offset, SEEK_SET) < 0)
 		return 0;
-	if (read(fd, &ch, 1) < 1)
+	if (read_nointr(fd, &ch, 1) < 1)
 		return 0;
 	return 1;
 }
@@ -118,7 +119,7 @@ static uint32_t get_blocks(int fd, int blocksize, uint32_t opt_blocks)
 
 	if (blocks > UINT32_MAX)
 	{
-		fprintf(stderr, "%s: Warning: Disk is too big, using only %lu blocks\n", appname, (unsigned long int)UINT32_MAX);
+		fprintf(stderr, "%s: Warning: Disk is too big, using only %"PRIu32" blocks\n", appname, UINT32_MAX);
 		return UINT32_MAX;
 	}
 
@@ -133,7 +134,8 @@ static void detect_blocksize(int fd, struct udf_disc *disc, int *blocksize)
 	if (ioctl(fd, BLKSSZGET, &size) != 0 || size <= 0)
 		return;
 
-	disc->blkssz = size;
+	if (!disc->blkssz)
+		disc->blkssz = size;
 
 	if (*blocksize != -1)
 		return;
@@ -235,7 +237,7 @@ static int is_removable_disk(int fd)
 	if (rem_fd < 0)
 		return -1;
 
-	ret = read(rem_fd, buf, sizeof(buf)-1);
+	ret = read_nointr(rem_fd, buf, sizeof(buf)-1);
 	close(rem_fd);
 
 	if (ret < 0)
@@ -264,6 +266,8 @@ static int write_func(struct udf_disc *disc, struct udf_extent *ext)
 	{
 		bufferlen = disc->blocksize;
 		buffer = calloc(bufferlen, 1);
+		if (buffer == NULL)
+			return -1;
 	}
 
 	if (!(ext->space_type & (USPACE|RESERVED)))
@@ -284,6 +288,8 @@ static int write_func(struct udf_disc *disc, struct udf_extent *ext)
 				{
 					bufferlen = (data->length + offset + disc->blocksize - 1) & ~(disc->blocksize - 1);
 					buffer = realloc(buffer, bufferlen);
+					if (buffer == NULL)
+						return -1;
 				}
 				memcpy(buffer + offset, data->buffer, data->length);
 				offset += data->length;
@@ -294,7 +300,7 @@ static int write_func(struct udf_disc *disc, struct udf_extent *ext)
 				memset(buffer + offset, 0x00, length - offset);
 			if (!(disc->flags & FLAG_NO_WRITE))
 			{
-				if (write(fd, buffer, length) != length)
+				if (write_nointr(fd, buffer, length) != length)
 					return -1;
 			}
 			desc = desc->next;
@@ -314,7 +320,7 @@ static int write_func(struct udf_disc *disc, struct udf_extent *ext)
 		{
 			if (!(disc->flags & FLAG_NO_WRITE))
 			{
-				if (write(fd, buffer, length) != length)
+				if (write_nointr(fd, buffer, length) != length)
 					return -1;
 			}
 		}
@@ -325,10 +331,11 @@ static int write_func(struct udf_disc *disc, struct udf_extent *ext)
 int main(int argc, char *argv[])
 {
 	struct udf_disc	disc;
-	struct stat stat;
+	struct udf_extent *ext;
 	char *filename;
 	char buf[128*3];
 	int fd;
+	int flags;
 	int create_new_file = 0;
 	int blocksize = -1;
 	int media;
@@ -341,21 +348,28 @@ int main(int argc, char *argv[])
 	if (fcntl(2, F_GETFL) < 0 && open("/dev/null", O_WRONLY) < 0)
 		_exit(1);
 
-	setlocale(LC_CTYPE, "");
 	appname = "mkudffs";
 
+	if (!setlocale(LC_CTYPE, ""))
+		fprintf(stderr, "%s: Error: Cannot set locale/codeset, fallback to default 7bit C ASCII\n", appname);
+
 	udf_init_disc(&disc);
 	parse_args(argc, argv, &disc, &filename, &create_new_file, &blocksize, &media);
 
 	if (disc.flags & FLAG_NO_WRITE)
 		printf("Note: Not writing to device, just simulating\n");
 
-	fd = open(filename, O_RDONLY);
+	if (!(disc.flags & FLAG_NO_WRITE))
+		flags = O_RDWR | O_EXCL;
+	else
+		flags = O_RDONLY | O_EXCL;
+
+	fd = open(filename, flags);
 	if (fd < 0)
 	{
 		if (errno != ENOENT)
 		{
-			fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, strerror(errno));
+			fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, (errno != EBUSY) ? strerror(errno) : "Device is mounted or mkudffs is already running");
 			exit(1);
 		}
 
@@ -365,63 +379,6 @@ int main(int argc, char *argv[])
 			exit(1);
 		}
 	}
-	else
-	{
-		int fd2;
-		int flags2;
-		char filename2[64];
-		const char *error;
-
-		if (create_new_file)
-		{
-			fprintf(stderr, "%s: Error: Cannot create new image file '%s': %s\n", appname, filename, strerror(EEXIST));
-			exit(1);
-		}
-
-		if (fstat(fd, &stat) != 0)
-		{
-			fprintf(stderr, "%s: Error: Cannot stat device '%s': %s\n", appname, filename, strerror(errno));
-			exit(1);
-		}
-
-		if (!(disc.flags & FLAG_NO_WRITE))
-			flags2 = O_RDWR;
-		else
-			flags2 = O_RDONLY;
-
-		if (snprintf(filename2, sizeof(filename2), "/proc/self/fd/%d", fd) >= (int)sizeof(filename2))
-		{
-			fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, strerror(ENAMETOOLONG));
-			exit(1);
-		}
-
-		// Re-open block device with O_EXCL mode which fails when device is already mounted
-		if (S_ISBLK(stat.st_mode))
-			flags2 |= O_EXCL;
-
-		fd2 = open(filename2, flags2);
-		if (fd2 < 0)
-		{
-			if (errno != ENOENT)
-			{
-				error = (errno != EBUSY) ? strerror(errno) : "Device is mounted or mkudffs is already running";
-				fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, error);
-				exit(1);
-			}
-
-			// Fallback to orignal filename when /proc is not available, but this introduce race condition between stat and open
-			fd2 = open(filename, flags2);
-			if (fd2 < 0)
-			{
-				error = (errno != EBUSY) ? strerror(errno) : "Device is mounted or mkudffs is already running";
-				fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, error);
-				exit(1);
-			}
-		}
-
-		close(fd);
-		fd = fd2;
-	}
 
 	if (fd >= 0)
 		detect_blocksize(fd, &disc, &blocksize);
@@ -433,13 +390,19 @@ int main(int argc, char *argv[])
 	}
 
 	disc.blocks = get_blocks(fd, disc.blocksize, disc.blocks);
+	if (disc.blocks == 0)
+	{
+		fprintf(stderr, "%s: Error: Device '%s' is empty\n", appname, filename);
+		exit(1);
+	}
+
 	disc.head->blocks = disc.blocks;
 	disc.write = write_func;
 	disc.write_data = &fd;
 
 	if (!(disc.flags & FLAG_BOOTAREA_MASK))
 	{
-		if (media != MEDIA_TYPE_HD)
+		if (media != MEDIA_TYPE_HD || disc.start_block)
 			disc.flags |= FLAG_BOOTAREA_PRESERVE;
 		else if (fd >= 0 && is_removable_disk(fd) == 0 && is_whole_disk(fd) == 1)
 			disc.flags |= FLAG_BOOTAREA_MBR;
@@ -457,9 +420,12 @@ int main(int argc, char *argv[])
 	len = gen_uuid_from_vol_set_ident(buf, disc.udf_pvd[0]->volSetIdent, 128);
 	printf("uuid=%s\n", buf);
 
-	printf("blocksize=%u\n", (unsigned int)disc.blocksize);
-	printf("blocks=%lu\n", (unsigned long int)disc.blocks);
-	printf("udfrev=%x.%02x\n", (unsigned int)(disc.udf_rev >> 8), (unsigned int)(disc.udf_rev & 0xFF));
+	printf("blocksize=%"PRIu32"\n", disc.blocksize);
+	printf("blocks=%"PRIu32"\n", disc.blocks);
+	printf("udfrev=%"PRIx16".%02"PRIx16"\n", disc.udf_rev >> 8, disc.udf_rev & 0xFF);
+
+	if (disc.start_block)
+		printf("startblock=%"PRIu32"\n", disc.start_block);
 
 	split_space(&disc);
 
@@ -470,7 +436,7 @@ int main(int argc, char *argv[])
 	setup_vds(&disc);
 
 	if (disc.vat_block)
-		printf("vatblock=%lu\n", (unsigned long int)disc.vat_block);
+		printf("vatblock=%"PRIu32"\n", disc.vat_block);
 
 	dump_space(&disc);
 
@@ -494,6 +460,13 @@ int main(int argc, char *argv[])
 			fprintf(stderr, "%s: Error: Cannot create new image file '%s': %s\n", appname, filename, strerror(errno));
 			exit(1);
 		}
+
+		// If creating new disk image file then shift all blocks to the beginning of the file
+		if (disc.start_block)
+		{
+			for (ext = disc.head; ext != NULL; ext = ext->next)
+				ext->start -= disc.start_block;
+		}
 	}
 
 	if (write_disc(&disc) < 0)
@@ -502,5 +475,20 @@ int main(int argc, char *argv[])
 		return 1;
 	}
 
+	if (!(disc.flags & FLAG_NO_WRITE))
+	{
+		if (fsync(fd) != 0)
+		{
+			fprintf(stderr, "%s: Error: Synchronization to device '%s' failed: %s\n", appname, filename, strerror(errno));
+			exit(1);
+		}
+	}
+
+	if (fd >= 0 && close(fd) != 0 && errno != EINTR)
+	{
+		fprintf(stderr, "%s: Error: Closing device '%s' failed: %s\n", appname, filename, strerror(errno));
+		exit(1);
+	}
+
 	return 0;
 }
