@@ -2,7 +2,7 @@
  * mkudffs.h
  *
  * Copyright (c) 2001-2002  Ben Fennema
- * Copyright (c) 2016-2018  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (c) 2016-2021  Pali Rohár <pali.rohar@gmail.com>
  * All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,17 +28,17 @@
 #include "osta_udf.h"
 #include "libudffs.h"
 
-#define CS0				0x00000001
-#define UDF_ID_APPLICATION		"*Linux mkudffs"
+#define UDF_ID_APPLICATION		"*Linux mkudffs " PACKAGE_VERSION
+#define UDF_ID_DEVELOPER		"*Linux UDFFS"
 
 #define DEFAULT_HD	0
 #define DEFAULT_CD	0
+#define DEFAULT_CDR	0
 #define DEFAULT_DVD	0
 #define DEFAULT_DVDRAM	0
 #define DEFAULT_WORM	1
 #define DEFAULT_MO	1
 #define DEFAULT_CDRW	2
-#define DEFAULT_CDR	3
 #define DEFAULT_BDR	3
 #define DEFAULT_DVDRW	4
 #define DEFAULT_DVDR	5
