@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (C) 2017-2021  Pali Rohár <pali.rohar@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -38,5 +38,7 @@ void parse_args(int, char *[], struct udf_disc *, char **);
 
 #define OPT_BLK_SIZE	0x2000
 #define OPT_VAT_BLOCK	0x2001
+#define OPT_START_BLOCK	0x2002
+#define OPT_LAST_BLOCK	0x2003
 
 #endif /* OPTIONS_H */
