@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2019  Pali Rohár <pali.rohar@gmail.com>
+ * Copyright (C) 2017-2021  Pali Rohár <pali.rohar@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -181,7 +181,6 @@ static void write_desc(int fd, struct udf_disc *disc, enum udf_space_type type,
 int main(int argc, char *argv[])
 {
 	struct udf_disc disc;
-	struct stat stat;
 	char *filename;
 	struct partitionDesc *pd;
 	struct logicalVolDesc *lvd;
@@ -189,7 +188,7 @@ int main(int argc, char *argv[])
 	struct domainIdentSuffix *dis;
 	size_t len;
 	int fd;
-	int fd2;
+	int flags;
 	int i;
 	char buf[256];
 	dstring new_lvid[128];
@@ -198,6 +197,11 @@ int main(int argc, char *argv[])
 	dstring new_fullvsid[128];
 	char new_uuid[17];
 	dstring new_vsid[128];
+	dstring new_owner[36];
+	dstring new_org[36];
+	dstring new_contact[36];
+	char new_appid[23];
+	char new_impid[23];
 	int force = 0;
 	int update = 0;
 	int update_pvd = 0;
@@ -212,9 +216,11 @@ int main(int argc, char *argv[])
 	if (fcntl(2, F_GETFL) < 0 && open("/dev/null", O_WRONLY) < 0)
 		_exit(1);
 
-	setlocale(LC_CTYPE, "");
 	appname = "udflabel";
 
+	if (!setlocale(LC_CTYPE, ""))
+		fprintf(stderr, "%s: Error: Cannot set locale/codeset, fallback to default 7bit C ASCII\n", appname);
+
 	memset(&disc, 0, sizeof(disc));
 
 	disc.head = calloc(1, sizeof(struct udf_extent));
@@ -224,6 +230,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
+	disc.start_block = (uint32_t)-1;
 	disc.flags = FLAG_LOCALE;
 	disc.tail = disc.head;
 	disc.head->space_type = USPACE;
@@ -233,6 +240,11 @@ int main(int argc, char *argv[])
 	memset(new_fsid, 0, sizeof(new_fsid));
 	memset(new_fullvsid, 0, sizeof(new_fullvsid));
 	memset(new_vsid, 0, sizeof(new_vsid));
+	memset(new_owner, 0, sizeof(new_owner));
+	memset(new_org, 0, sizeof(new_org));
+	memset(new_contact, 0, sizeof(new_contact));
+	memset(new_appid, 0, sizeof(new_appid));
+	memset(new_impid, 0, sizeof(new_impid));
 
 	new_lvid[0] = 0xFF;
 	new_vid[0] = 0xFF;
@@ -240,8 +252,13 @@ int main(int argc, char *argv[])
 	new_fullvsid[0] = 0xFF;
 	new_uuid[0] = 0;
 	new_vsid[0] = 0xFF;
+	new_owner[0] = 0xFF;
+	new_org[0] = 0xFF;
+	new_contact[0] = 0xFF;
+	new_appid[0] = (char)-1;
+	new_impid[0] = (char)-1;
 
-	parse_args(argc, argv, &disc, &filename, &force, new_lvid, new_vid, new_fsid, new_fullvsid, new_uuid, new_vsid);
+	parse_args(argc, argv, &disc, &filename, &force, new_lvid, new_vid, new_fsid, new_fullvsid, new_uuid, new_vsid, new_owner, new_org, new_contact, new_appid, new_impid);
 
 	if (disc.flags & FLAG_NO_WRITE)
 		printf("Note: Not writing to device, just simulating\n");
@@ -259,72 +276,40 @@ int main(int argc, char *argv[])
 	if (new_fsid[0] != 0xFF)
 		update_fsd = 1;
 
-	if (new_uuid[0] || new_vsid[0] != 0xFF || new_fullvsid[0] != 0xFF)
+	if (new_owner[0] != 0xFF || new_org[0] != 0xFF || new_contact[0] != 0xFF)
+		update_iuvd = 1;
+
+	if (new_uuid[0] || new_vsid[0] != 0xFF || new_fullvsid[0] != 0xFF || new_appid[0] != (char)-1 || new_impid[0] != (char)-1)
 		update_pvd = 1;
 
 	if (update_pvd || update_lvd || update_iuvd || update_fsd)
 		update = 1;
 
-	fd = open(filename, O_RDONLY);
-	if (fd < 0)
+	if (update && !(disc.flags & FLAG_NO_WRITE))
+		flags = O_RDWR | O_EXCL;
+	else
+		flags = O_RDONLY | O_EXCL;
+
+	fd = open(filename, flags);
+	if (fd < 0 && errno == EBUSY)
 	{
-		fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, strerror(errno));
-		exit(1);
-	}
-
-	if (fstat(fd, &stat) != 0)
-	{
-		fprintf(stderr, "%s: Error: Cannot stat device '%s': %s\n", appname, filename, strerror(errno));
-		exit(1);
-	}
-
-	if (update)
-	{
-		int flags2;
-		char filename2[64];
-		const char *error;
-
-		if (!(disc.flags & FLAG_NO_WRITE))
-			flags2 = O_RDWR;
-		else
-			flags2 = O_RDONLY;
-
-		if (snprintf(filename2, sizeof(filename2), "/proc/self/fd/%d", fd) >= (int)sizeof(filename2))
+		if (update)
 		{
-			fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, strerror(ENAMETOOLONG));
+			fprintf(stderr, "%s: Error: Cannot open device '%s': Device is busy, maybe mounted?\n", appname, filename);
 			exit(1);
 		}
-
-		// Re-open block device with O_EXCL mode which fails when device is already mounted
-		if (S_ISBLK(stat.st_mode))
-			flags2 |= O_EXCL;
-
-		fd2 = open(filename2, flags2);
-		if (fd2 < 0)
-		{
-			// Fallback to orignal filename when /proc is not available, but this introduce race condition between stat and open
-			if (errno == ENOENT)
-				fd2 = open(filename, flags2);
-			if (fd2 < 0)
-			{
-				error = (errno != EBUSY) ? strerror(errno) : "Device is busy, maybe mounted?";
-				fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, error);
-				exit(1);
-			}
-		}
-
-		close(fd);
-		fd = fd2;
+		flags &= ~O_EXCL;
+		fd = open(filename, flags);
 	}
-	else if (S_ISBLK(stat.st_mode))
+	if (fd < 0)
 	{
-		fd2 = open(filename, O_RDONLY|O_EXCL);
-		if (fd2 >= 0)
-			close(fd2);
-		else if (errno == EBUSY)
-			fprintf(stderr, "%s: Warning: Device '%s' is busy, %s may report bogus information\n", appname, filename, appname);
+		fprintf(stderr, "%s: Error: Cannot open device '%s': %s\n", appname, filename, (errno != EBUSY) ? strerror(errno) : "Device is busy, maybe mounted?");
+		exit(1);
 	}
 
+	if (!(flags & O_EXCL))
+		fprintf(stderr, "%s: Warning: Device '%s' is busy, %s may report bogus information\n", appname, filename, appname);
+
 	disc.blksize = get_size(fd);
 	disc.blkssz = get_sector_size(fd);
 
@@ -348,6 +333,12 @@ int main(int argc, char *argv[])
 			exit(1);
 		}
 
+		if (lvd->descCharSet.charSetType != UDF_CHAR_SET_TYPE || strncmp((const char *)lvd->descCharSet.charSetInfo, UDF_CHAR_SET_INFO, sizeof(lvd->descCharSet.charSetInfo)) != 0)
+		{
+			fprintf(stderr, "%s: Error: Label is not encoded in OSTA Unicode dstring\n", appname);
+			exit(1);
+		}
+
 		len = decode_string(&disc, lvd->logicalVolIdent, buf, sizeof(lvd->logicalVolIdent), sizeof(buf));
 		if (len == (size_t)-1)
 		{
@@ -388,7 +379,11 @@ int main(int argc, char *argv[])
 	{
 		case PD_ACCESS_TYPE_OVERWRITABLE:
 		case PD_ACCESS_TYPE_REWRITABLE:
+			break;
+
 		case PD_ACCESS_TYPE_NONE: /* Pseudo OverWrite */
+			if (force)
+				fprintf(stderr, "%s: Warning: Trying to overwrite pseudo-overwrite partition\n", appname);
 			break;
 
 		case PD_ACCESS_TYPE_WRITE_ONCE:
@@ -440,6 +435,13 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 
+	/* TODO: Pseudo OverWrite mode */
+	if (le32_to_cpu(pd->accessType) == PD_ACCESS_TYPE_NONE && !force)
+	{
+		fprintf(stderr, "%s: Error: Updating pseudo-overwrite partition is not supported yet\n", appname);
+		exit(1);
+	}
+
 	if (disc.udf_lvd[0] && update_lvd)
 	{
 		dis = (struct domainIdentSuffix *)disc.udf_lvd[0]->domainIdent.identSuffix;
@@ -492,6 +494,12 @@ int main(int argc, char *argv[])
 	{
 		if (!new_uuid[0] || new_vsid[0] == 0xFF)
 		{
+			if (disc.udf_pvd[0]->descCharSet.charSetType != UDF_CHAR_SET_TYPE || strncmp((const char *)disc.udf_pvd[0]->descCharSet.charSetInfo, UDF_CHAR_SET_INFO, sizeof(disc.udf_pvd[0]->descCharSet.charSetInfo)) != 0)
+			{
+				fprintf(stderr, "%s: Error: Volume Set Identifier is not encoded in OSTA Unicode dstring\n", appname);
+				fprintf(stderr, "%s: Error: In this case it is needed to specify both --vsid and --uuid\n", appname);
+				exit(1);
+			}
 			memset(buf, 0, sizeof(buf));
 			len = gen_uuid_from_vol_set_ident(buf, disc.udf_pvd[0]->volSetIdent, sizeof(disc.udf_pvd[0]->volSetIdent));
 			if (len < 16)
@@ -585,12 +593,27 @@ int main(int argc, char *argv[])
 		memset(buf, 0, sizeof(buf));
 		decode_string(&disc, new_lvid, buf, sizeof(new_lvid), sizeof(buf));
 		printf("Using new Logical Volume Identifier: %s\n", buf);
+		memset(&disc.udf_lvd[0]->descCharSet, 0, sizeof(disc.udf_lvd[0]->descCharSet));
+		memset(&disc.udf_lvd[1]->descCharSet, 0, sizeof(disc.udf_lvd[1]->descCharSet));
+		disc.udf_lvd[0]->descCharSet.charSetType = UDF_CHAR_SET_TYPE;
+		disc.udf_lvd[1]->descCharSet.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)disc.udf_lvd[0]->descCharSet.charSetInfo, UDF_CHAR_SET_INFO);
+		strcpy((char *)disc.udf_lvd[1]->descCharSet.charSetInfo, UDF_CHAR_SET_INFO);
 		memcpy(disc.udf_lvd[0]->logicalVolIdent, new_lvid, sizeof(new_lvid));
 		memcpy(disc.udf_lvd[1]->logicalVolIdent, new_lvid, sizeof(new_lvid));
 		iuvdiu = (struct impUseVolDescImpUse *)disc.udf_iuvd[0]->impUse;
+		memset(&iuvdiu->LVICharset, 0, sizeof(iuvdiu->LVICharset));
+		iuvdiu->LVICharset.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)iuvdiu->LVICharset.charSetInfo, UDF_CHAR_SET_INFO);
 		memcpy(iuvdiu->logicalVolIdent, new_lvid, sizeof(new_lvid));
 		iuvdiu = (struct impUseVolDescImpUse *)disc.udf_iuvd[1]->impUse;
+		memset(&iuvdiu->LVICharset, 0, sizeof(iuvdiu->LVICharset));
+		iuvdiu->LVICharset.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)iuvdiu->LVICharset.charSetInfo, UDF_CHAR_SET_INFO);
 		memcpy(iuvdiu->logicalVolIdent, new_lvid, sizeof(new_lvid));
+		memset(&disc.udf_fsd->logicalVolIdentCharSet, 0, sizeof(disc.udf_fsd->logicalVolIdentCharSet));
+		disc.udf_fsd->logicalVolIdentCharSet.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)disc.udf_fsd->logicalVolIdentCharSet.charSetInfo, UDF_CHAR_SET_INFO);
 		memcpy(disc.udf_fsd->logicalVolIdent, new_lvid, sizeof(new_lvid));
 	}
 
@@ -599,6 +622,12 @@ int main(int argc, char *argv[])
 		memset(buf, 0, sizeof(buf));
 		decode_string(&disc, new_vid, buf, sizeof(new_vid), sizeof(buf));
 		printf("Using new Volume Identifier: %s\n", buf);
+		memset(&disc.udf_pvd[0]->descCharSet, 0, sizeof(disc.udf_pvd[0]->descCharSet));
+		memset(&disc.udf_pvd[1]->descCharSet, 0, sizeof(disc.udf_pvd[1]->descCharSet));
+		disc.udf_pvd[0]->descCharSet.charSetType = UDF_CHAR_SET_TYPE;
+		disc.udf_pvd[1]->descCharSet.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)disc.udf_pvd[0]->descCharSet.charSetInfo, UDF_CHAR_SET_INFO);
+		strcpy((char *)disc.udf_pvd[1]->descCharSet.charSetInfo, UDF_CHAR_SET_INFO);
 		memcpy(disc.udf_pvd[0]->volIdent, new_vid, sizeof(new_vid));
 		memcpy(disc.udf_pvd[1]->volIdent, new_vid, sizeof(new_vid));
 	}
@@ -608,9 +637,80 @@ int main(int argc, char *argv[])
 		memset(buf, 0, sizeof(buf));
 		decode_string(&disc, new_fsid, buf, sizeof(new_fsid), sizeof(buf));
 		printf("Using new File Set Identifier: %s\n", buf);
+		memset(&disc.udf_fsd->fileSetCharSet, 0, sizeof(disc.udf_fsd->fileSetCharSet));
+		disc.udf_fsd->fileSetCharSet.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)disc.udf_fsd->fileSetCharSet.charSetInfo, UDF_CHAR_SET_INFO);
 		memcpy(disc.udf_fsd->fileSetIdent, new_fsid, sizeof(new_fsid));
 	}
 
+	if (new_owner[0] != 0xFF)
+	{
+		memset(buf, 0, sizeof(buf));
+		decode_string(&disc, new_owner, buf, sizeof(new_owner), sizeof(buf));
+		printf("Using new Owner name: %s\n", buf);
+		iuvdiu = (struct impUseVolDescImpUse *)disc.udf_iuvd[0]->impUse;
+		memset(&iuvdiu->LVICharset, 0, sizeof(iuvdiu->LVICharset));
+		iuvdiu->LVICharset.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)iuvdiu->LVICharset.charSetInfo, UDF_CHAR_SET_INFO);
+		memcpy(iuvdiu->LVInfo1, new_owner, sizeof(new_owner));
+		iuvdiu = (struct impUseVolDescImpUse *)disc.udf_iuvd[1]->impUse;
+		memset(&iuvdiu->LVICharset, 0, sizeof(iuvdiu->LVICharset));
+		iuvdiu->LVICharset.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)iuvdiu->LVICharset.charSetInfo, UDF_CHAR_SET_INFO);
+		memcpy(iuvdiu->LVInfo1, new_owner, sizeof(new_owner));
+	}
+
+	if (new_org[0] != 0xFF)
+	{
+		memset(buf, 0, sizeof(buf));
+		decode_string(&disc, new_org, buf, sizeof(new_org), sizeof(buf));
+		printf("Using new Oranization name: %s\n", buf);
+		iuvdiu = (struct impUseVolDescImpUse *)disc.udf_iuvd[0]->impUse;
+		memset(&iuvdiu->LVICharset, 0, sizeof(iuvdiu->LVICharset));
+		iuvdiu->LVICharset.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)iuvdiu->LVICharset.charSetInfo, UDF_CHAR_SET_INFO);
+		memcpy(iuvdiu->LVInfo2, new_org, sizeof(new_org));
+		iuvdiu = (struct impUseVolDescImpUse *)disc.udf_iuvd[1]->impUse;
+		memset(&iuvdiu->LVICharset, 0, sizeof(iuvdiu->LVICharset));
+		iuvdiu->LVICharset.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)iuvdiu->LVICharset.charSetInfo, UDF_CHAR_SET_INFO);
+		memcpy(iuvdiu->LVInfo2, new_org, sizeof(new_org));
+	}
+
+	if (new_contact[0] != 0xFF)
+	{
+		memset(buf, 0, sizeof(buf));
+		decode_string(&disc, new_contact, buf, sizeof(new_contact), sizeof(buf));
+		printf("Using new Contact information: %s\n", buf);
+		iuvdiu = (struct impUseVolDescImpUse *)disc.udf_iuvd[0]->impUse;
+		memset(&iuvdiu->LVICharset, 0, sizeof(iuvdiu->LVICharset));
+		iuvdiu->LVICharset.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)iuvdiu->LVICharset.charSetInfo, UDF_CHAR_SET_INFO);
+		memcpy(iuvdiu->LVInfo3, new_contact, sizeof(new_contact));
+		iuvdiu = (struct impUseVolDescImpUse *)disc.udf_iuvd[1]->impUse;
+		memset(&iuvdiu->LVICharset, 0, sizeof(iuvdiu->LVICharset));
+		iuvdiu->LVICharset.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)iuvdiu->LVICharset.charSetInfo, UDF_CHAR_SET_INFO);
+		memcpy(iuvdiu->LVInfo3, new_contact, sizeof(new_contact));
+	}
+
+	if (new_appid[0] != (char)-1)
+	{
+		printf("Using new Application Identifier: %.*s\n", (int)sizeof(new_appid), new_appid);
+		memset(&disc.udf_pvd[0]->appIdent, 0, sizeof(disc.udf_pvd[0]->appIdent));
+		memset(&disc.udf_pvd[1]->appIdent, 0, sizeof(disc.udf_pvd[1]->appIdent));
+		memcpy(disc.udf_pvd[0]->appIdent.ident, new_appid, sizeof(new_appid));
+		memcpy(disc.udf_pvd[1]->appIdent.ident, new_appid, sizeof(new_appid));
+	}
+
+	if (new_impid[0] != (char)-1)
+	{
+		printf("Using new Implementation Identifier: %.*s\n", (int)sizeof(new_impid), new_impid);
+		/* Do not erase identSuffix which contains Operating System Class and Id */
+		memcpy(disc.udf_pvd[0]->impIdent.ident, new_impid, sizeof(new_impid));
+		memcpy(disc.udf_pvd[1]->impIdent.ident, new_impid, sizeof(new_impid));
+	}
+
 	if (new_fullvsid[0] != 0xFF)
 	{
 		memset(buf, 0, sizeof(buf));
@@ -642,6 +742,13 @@ int main(int argc, char *argv[])
 		decode_string(&disc, new_fullvsid, buf, sizeof(new_fullvsid), sizeof(buf));
 		printf("Using new full Volume Set Identifier: %s\n", buf);
 
+		memset(&disc.udf_pvd[0]->descCharSet, 0, sizeof(disc.udf_pvd[0]->descCharSet));
+		memset(&disc.udf_pvd[1]->descCharSet, 0, sizeof(disc.udf_pvd[1]->descCharSet));
+		disc.udf_pvd[0]->descCharSet.charSetType = UDF_CHAR_SET_TYPE;
+		disc.udf_pvd[1]->descCharSet.charSetType = UDF_CHAR_SET_TYPE;
+		strcpy((char *)disc.udf_pvd[0]->descCharSet.charSetInfo, UDF_CHAR_SET_INFO);
+		strcpy((char *)disc.udf_pvd[1]->descCharSet.charSetInfo, UDF_CHAR_SET_INFO);
+
 		memcpy(disc.udf_pvd[0]->volSetIdent, new_fullvsid, sizeof(new_fullvsid));
 		memcpy(disc.udf_pvd[1]->volSetIdent, new_fullvsid, sizeof(new_fullvsid));
 	}
